from collections import OrderedDict

from pyramid.httpexceptions import HTTPNotFound
from pyramid.security import (
    Allow,
    ALL_PERMISSIONS,
    DENY_ALL,
)

from pyramlson import api_service, api_method


class Book(object):
    """ A simcple book class """

    def __init__(self, id, title, author, isbn=None):
        self.id = id
        self.title = title
        self.author = author
        selfisbn = isbn

    @classmethod
    def from_dict(klass, id, data):
        return klass(id, data['title'], data['author'], data.get('isbn', None))

    @classmethod
    def from_etree(klass, id, tree):
        return klass(id, data['title'], data['author'], data.get('isbn', None))

BOOKS = OrderedDict()
BOOKS[123] = {"id": 123,
    "title": "Dune",
    "author": "Frank Herbert",
    "isbn": "98765"
}
BOOKS[456] = {
    "id": 456,
    "title": "Hyperion Cantos",
    "author": "Dan Simmons",
    "isbn": "56789"
}

def get_book(book_id):
    bid = int(book_id)
    if bid not in BOOKS:
        raise BookNotFound("Book with id {} could not be found.".format(book_id))
    return BOOKS[bid]

class BookNotFound(HTTPNotFound):
    pass

@api_service('/books')
class BooksResource(object):

    __acl__ = [
        (Allow, 'api-user-reader', ('view', )),
        (Allow, 'api-user-writer', ('view', 'create', 'update')),
        (Allow, 'admin', ALL_PERMISSIONS),
        DENY_ALL,
    ]

    def __init__(self, request):
        self.request = request

    @api_method('post', permission='create')
    def create_record(self, data):
        bid = int(data["id"])
        BOOKS[bid] = data
        return BOOKS[bid]

    @api_method('get', permission='view')
    def get_all(self, sort_by='id', sort_reversed=False, offset=0, limit=10):
        return list(BOOKS.values())


@api_service('/books/{bookId}')
class BookResource(object):

    __acl__ = [
        (Allow, 'api-user-reader', ('view', )),
        (Allow, 'api-user-writer', ('view', 'create', 'update')),
        (Allow, 'admin', ALL_PERMISSIONS),
        DENY_ALL,
    ]

    def __init__(self, request):
        self.request = request

    @api_method('get', permission='view')
    def get_one(self, book_id):
        return get_book(book_id)

    @api_method('put',
            permission='update',
            returns=200)
    def update(self, book_id, data):
        book = get_book(book_id)
        book.update(data)
        return dict(success=True)

    @api_method('delete',
            permission='delete',
            returns=204)
    def delete(self, book_id):
        book = get_book(book_id)
        BOOKS.pop(book["id"])


@api_service('/books/some/other/things')
class SomeOtherThings(object):

    def __init__(self, request):
        self.request = request

    @api_method('get')
    def things(self, thing_type=None):
        return dict(thing_type=thing_type)



from pyramlson import api_service, api_method

@api_service('/foo')
class ErrorsResource(object):

    def __init__(self, request):
        self.request = request

    @api_method('get')
    def get(self):
        raise Exception()


@api_service('/foo/{one}/{two}')
class ErrorsOneTwoResource(object):

    def __init__(self, request):
        self.request = request

    @api_method('get')
    def foo(self, one, two):
        return dict(one=one, two=two)



import os
import unittest

from pyramid import testing
from pyramid.config import Configurator
from pyramid.authentication import BasicAuthAuthenticationPolicy
from pyramid.authorization import ACLAuthorizationPolicy

from .base import DATA_DIR
from .resource import BOOKS

def dummy_check(username, password, request):
    if username == 'somebody' and password == 'bar':
        return [] # authorized without roles
    if username == 'writer' and password == 'bar':
        return ['api-user-writer']
    if username == 'admin' and password == 'bar':
        return ['admin']

class Root:
    __name__ = 'root'
    __parent__ = None

class ErrorsTests(unittest.TestCase):

    def setUp(self):
        settings = {
            'pyramlson.apidef_path': os.path.join(DATA_DIR, 'test-api.raml'),
        }
        auth_policy = BasicAuthAuthenticationPolicy(dummy_check, 'TEST REALM')
        self.config = testing.setUp(settings=settings)
        self.config.set_authorization_policy(ACLAuthorizationPolicy())
        self.config.set_authentication_policy(auth_policy)
        self.config.include('pyramlson')
        self.config.scan('.resource')
        from webtest import TestApp
        self.testapp = TestApp(self.config.make_wsgi_app())

    def tearDown(self):
        testing.tearDown()

    def test_general_notfound(self):
        r = self.testapp.get('/foo', status=404)
        self.assertEquals(r.json_body['message'], '/foo')
        self.assertEquals(r.json_body['success'], False)

    def test_unauthorized(self):
        r = self.testapp.get('/api/v1/books', status=401)
        self.assertEquals(r.json_body['message'], 'You must login to perform this action.')
        self.assertEquals(r.json_body['success'], False)

    def test_bad_roles(self):
        self.testapp.authorization = ('Basic', ('somebody', 'bar'))
        r = self.testapp.get('/api/v1/books', status=403)
        self.assertEquals(r.json_body['message'], 'You are not allowed to perform this action.')
        self.assertEquals(r.json_body['success'], False)

class ErrorsWithDebugTests(unittest.TestCase):
    def setUp(self):
        settings = {
            'pyramlson.apidef_path': os.path.join(DATA_DIR, 'test-errors-api.raml'),
            'pyramlson.debug': 'true',
        }
        self.config = testing.setUp(settings=settings)
        self.config.include('pyramlson')
        self.config.scan('.error_resource')
        from webtest import TestApp
        self.testapp = TestApp(self.config.make_wsgi_app())

    def test_traceback(self):
        r = self.testapp.get('/api/v1/foo', status=500)
        self.assertEquals(r.json_body['success'], False)
        self.assertTrue('traceback' in r.json_body)

class UnknownErrorTests(unittest.TestCase):
    def setUp(self):
        settings = {
            'pyramlson.apidef_path': os.path.join(DATA_DIR, 'test-errors-api.raml'),
            'pyramlson.debug': 'true',
        }
        self.config = testing.setUp(settings=settings)
        self.config.include('pyramlson')
        self.config.scan('.error_resource')
        from webtest import TestApp
        self.testapp = TestApp(self.config.make_wsgi_app())

    def test_unknown_error(self):
        r = self.testapp.get('/api/v1/foo', status=500)
        self.assertEquals(r.json_body['success'], False)
        self.assertEquals(r.json_body['message'], "Unknown error")



import json
import os
import unittest

from pyramid import testing

from pyramlson import apidef

from .base import DATA_DIR

def get_api(which='test-api.raml'):
    path = os.path.join(DATA_DIR, which)
    return apidef.RamlApiDefinition(path)

def test_base():
    api = get_api()
    assert api.raml != None
    assert api.base_uri == 'http://{apiUri}/api/v1'
    assert api.base_path == '/api/v1'

def test_notexisting_resources():
    api = get_api()
    assert len(list(api.get_resources('/foo'))) == 0
    assert list(api.get_resources('/books'))[0] == api.raml.resources[0]

def test_filtered_resources():
    api = get_api('test-errors-api.raml')
    assert len(list(api.get_resources('/foo'))) == 1
    assert len(list(api.get_resources('/bar'))) == 1

def test_all_resources():
    api = get_api('test-errors-api.raml')
    assert len(list(api.get_resources())) == 3

def test_schema():
    api = get_api()
    assert api.get_schema(None) is None

    resource = list(api.get_resources('/books'))[1]
    body = resource.body[0]
    schema = json.load(open(os.path.join(DATA_DIR, 'schemas', 'BookRecord.json')))
    assert api.get_schema(body) == schema

def test_existing_traits():
    api = get_api()
    trait = api.get_trait('sorted')
    assert trait is not None
    assert str(trait.description) == 'A sorted collection resource'
    assert len(trait.query_params) == len(api.raml.traits[0].query_params)

    trait = api.get_trait('paged')
    assert trait is not None
    assert str(trait.description) == 'A paged collection resource'
    assert len(trait.query_params) == len(api.raml.traits[1].query_params)

def test_notexisting_traits():
    api = get_api()
    trait = api.get_trait('foo')
    assert trait is None

def test_empty_traits():
    api = get_api('test-errors-api.raml')
    assert api.get_trait('foo') is None

class TestMissingRaml(unittest.TestCase):
    def test_missing_raml(self):
        # not providing a path to RAML specs file in settings['pyramlson.apidef_path']
        # must raise a ValueError
        config = testing.setUp()
        self.assertRaises(ValueError, config.include, 'pyramlson')



# coding: utf-8
"""
Main pyramlson decorators: api_service and api_method
"""
import re
import os
import logging

from inspect import getmembers
from collections import namedtuple, defaultdict

import venusian

from pyramid.path import (
    AssetResolver,
    DottedNameResolver
)
from pyramid.httpexceptions import (
    HTTPBadRequest,
    HTTPInternalServerError,
    HTTPNoContent,
)
from pyramid.interfaces import IExceptionResponse

from .apidef import IRamlApiDefinition
from .utils import prepare_json_body, render_view

LOG = logging.getLogger(__name__)


DEFAULT_METHOD_MAP = {
    'get': 200,
    'post': 200,
    'put': 201,
    'delete': 204,
    'options': 200,
    'patch': 200,
}

MethodRestConfig = namedtuple('MethodRestConfig', [
    'http_method',
    'permission',
    'returns'
])


MARKER = object()

class api_method(object):
    # pylint: disable=invalid-name

    def __init__(self, http_method, permission=None, returns=None):
        """Configure a resource method corresponding with a RAML resource path

        This decorator must be used to declare REST resources.

        :param http_method: The HTTP method this method maps to.

        :param permission: Permission for this method.

        :param returns: A custom HTTP code to return in case of success.

            Configure the HTTP code to return when the method call was successful.
            Per default the codes are expected to match the configured HTTP method:
                - GET/POST/PATCH: 200
                - PUT: 201
                - DELETE: 204

        """
        self.http_method = http_method
        self.permission = permission
        self.returns = returns if returns is not None else DEFAULT_METHOD_MAP[self.http_method]

    def __call__(self, method):
        method._rest_config = MethodRestConfig(
            self.http_method,
            self.permission,
            self.returns
        )
        return method


class api_service(object):
    """Configures a resource by its REST path.

    This decorator configures a class as a REST resource. All endpoints
    must be defined in a RAML file.
    """
    # pylint: disable=invalid-name

    def __init__(self, resource_path):
        LOG.debug("Resource path: %s", resource_path)
        self.resource_path = resource_path
        self.resources = []
        self.apidef = None
        self.cls = None
        self.module = None

    def callback(self, scanner, name, cls):
        config = scanner.config.with_package(self.module)
        self.apidef = config.registry.queryUtility(IRamlApiDefinition)
        self.create_route(config)
        LOG.debug("registered routes with base route '%s'", self.apidef.base_path)
        self.create_views(config)

    def create_route(self, config):
        LOG.debug("Creating route for %s", self.resource_path)
        supported_methods = []
        route_name = None

        # Find all methods for this resource path
        for resource in self.apidef.get_resources(self.resource_path):
            if route_name is None:
                path = self.resource_path
                if self.apidef.base_path:
                    path = "{}{}".format(self.apidef.base_path, path)
                route_name = "{}-{}".format(resource.display_name, path)

            method = resource.method.upper()
            self.resources.append((route_name, method, resource, None))
            supported_methods.append(method)

        # Add one route for all the methods at this resource path
        if supported_methods:
            LOG.debug("Registering route with path %s", path)
            config.add_route(route_name, path, factory=self.cls)
            # add a default OPTIONS view if none was defined by the resource
            opts_meth = 'OPTIONS'
            if opts_meth not in supported_methods:
                methods = supported_methods + [opts_meth]
                self.resources.append((
                    route_name,
                    'OPTIONS',
                    resource,
                    create_options_view(methods)
                ))

    def create_views(self, config):
        for (route_name, method, resource, default_view) in self.resources:
            LOG.debug("Creating view %s %s", route_name, method)
            if default_view:
                config.add_view(
                    default_view,
                    route_name=route_name,
                    request_method=method
                )
            else:
                (view, permission) = self.create_view(resource)
                LOG.debug(
                    "Registering view %s for route name '%s', resource '%s', method '%s'",
                    view,
                    route_name,
                    resource,
                    method
                )
                config.add_view(
                    view,
                    route_name=route_name,
                    request_method=method,
                    permission=permission
                )

    def __call__(self, cls):
        self.cls = cls
        info = venusian.attach(cls, self.callback, 'pyramid', depth=1)
        self.module = info.module
        return cls

    def create_view(self, resource):
        (meth, cfg) = self.get_service_class_method(resource)
        LOG.debug("Got method %s for resource %s", meth, resource)
        if not meth:
            msg = "Could not find a method in class {} suitable for resource {}.".format(
                self.cls,
                resource
            )
            raise ValueError(msg)
        transform = self.apidef.args_transform_cb
        transform = transform if callable(transform) else lambda arg: arg
        def view(context, request):
            required_params = [context]
            optional_params = dict()
            # URI parameters have the highest prio
            if resource.uri_params:
                for param in resource.uri_params:
                    param_value = request.matchdict[param.name]
                    validate_param(param, param_value)
                    # pyramid router makes sure the URI params are all
                    # set, otherwise the view isn't called all, because
                    # a NotFound error is triggered before the request
                    # can be routed to this view
                    required_params.append(param_value)
            # If there's a body defined - include it before traits or query params
            if resource.body:
                required_params.append(prepare_json_body(request, resource.body))
            if resource.query_params:
                for param in resource.query_params:
                    param_value = request.params.get(param.name, MARKER)
                    if param_value is not MARKER:
                        validate_param(param, param_value)
                    else:
                        param_value = param.default
                    # query params are always named (i.e. not positional)
                    # so they effectively become keyword agruments in a
                    # method call, we just make sure they are present
                    # in the request if marked as 'required'
                    if param.required and param.name not in request.params:
                        raise HTTPBadRequest("{} ({}) is required".format(param.name, param.type))
                    else:
                        optional_params[transform(param.name)] = param_value
            result = meth(*required_params, **optional_params)
            return render_view(request, result, cfg.returns)

        return (view, cfg.permission)

    def get_service_class_method(self, resource):
        rel_path = resource.path[len(self.resource_path):]
        LOG.debug("Relative path for %s: '%s'", resource, rel_path)
        http_method = resource.method.lower()
        for (_, member) in getmembers(self.cls):
            if not hasattr(member, '_rest_config'):
                continue
            cfg = member._rest_config  # pylint: disable=protected-access
            if (cfg.http_method.lower() == http_method) and callable(member):
                return (member, cfg)
        return (None, None)

def create_options_view(supported_methods):
    """ Create a view callable for the OPTIONS request """
    def view(context, request):  # pylint: disable=unused-argument,missing-docstring
        response = HTTPNoContent()
        response.headers['Access-Control-Allow-Methods'] =\
            ', '.join(supported_methods)
        return response
    return view


def includeme(config):
    """Configure basic RAML REST settings for a Pyramid application.

    You should not call this function directly, but use
    :py:func:`pyramid.config.Configurator.include` to initialise
    the RAML routing.

    .. code-block:: python
       :linenos:
       config = Configurator()
       config.include('pyramlson')
    """
    from pyramlson.apidef import RamlApiDefinition
    settings = config.registry.settings
    settings['pyramlson.debug'] = \
            settings.get('debug_all') or \
            settings.get('pyramid.debug_all') or \
            settings.get('pyramlson.debug')
    config.add_view('pyramlson.error.generic', context=Exception, renderer='json')
    config.add_view('pyramlson.error.http_error', context=IExceptionResponse, renderer='json')
    config.add_notfound_view('pyramlson.error.notfound', renderer='json')
    config.add_forbidden_view('pyramlson.error.forbidden', renderer='json')

    if 'pyramlson.apidef_path' not in settings:
        raise ValueError("Cannot create RamlApiDefinition without a RAML file.")


    args_transform_cb = None
    if 'pyramlson.arguments_transformation_callback' in settings:
        args_transform_cb = DottedNameResolver().maybe_resolve(
            settings['pyramlson.arguments_transformation_callback']
        )

    res = AssetResolver()
    apidef_path = res.resolve(settings['pyramlson.apidef_path'])
    apidef = RamlApiDefinition(apidef_path.abspath(), args_transform_cb=args_transform_cb)
    config.registry.registerUtility(apidef, IRamlApiDefinition)


def validate_param(param, value):
    """ Validate input parameters """
    # FIXME: add more checks
    if param.type == 'integer':
        try:
            int(value)
        except ValueError:
            msg = "Malformed parameter '{}', expected integer, got '{}'".format(
                param.name, value
            )
            raise HTTPBadRequest(msg)
    if param.type == 'string':
        if param.enum and value not in param.enum:
            raise HTTPBadRequest("Malformed parameter '{}', expected one of {}, got '{}'".format(param.name, ', '.join(param.enum), value))
        if param.pattern:
            result = re.search(param.pattern, value)
            if not result:
                raise HTTPBadRequest("Malformed parameter '{}', expected pattern {}, got '{}'".format(param.name, param.pattern, value))
        if param.min_length and len(value) < param.min_length:
            msg = "Malformed parameter '{}', expected minimum length is {}, got {}"
            raise HTTPBadRequest(msg.format(
                param.name,
                param.min_length,
                len(value)
            ))
        if param.max_length and len(value) > param.max_length:
            msg = "Malformed parameter '{}', expected maximum length is {}, got {}"
            raise HTTPBadRequest(msg.format(
                param.name,
                param.max_length,
                len(value)
            ))
__all__ = ['api_method', 'api_service']



import logging
import traceback
from pyramid.httpexceptions import HTTPNotFound
from pyramid.security import unauthenticated_userid


log = logging.getLogger(__name__)


def err_dict(message):
    return dict(success=False, message=message)

def generic(context, request):
    log.error("error.generic -- context: \"{}\"".format(context))
    tb = ''.join(traceback.format_exception(*request.exc_info))
    log.error("error.generic -- traceback: \"{}\"".format(tb))
    request.response.status_int = 500
    try:
        response = err_dict(context.args[0])
    except IndexError:
        response = err_dict('Unknown error')
    if request.registry.settings.get('pyramlson.debug'):
        response['traceback'] = tb
    return response


def http_error(context, request):
    log.info("error.http_error -- context: \"{}\"".format(context))
    request.response.status = context.status
    for (header, value) in context.headers.items():
        if header in {'Content-Type', 'Content-Length'}:
            continue
        request.response.headers[header] = value
    if context.message:
        return err_dict(context.message)
    else:
        return err_dict(context.status)


def notfound(context, request):
    log.info("error.notfound -- context: \"{}\"".format(context))
    message = 'Resource not found'
    if isinstance(context, HTTPNotFound):
        if context.content_type == 'application/json':
            return context
        elif context.detail:
            message = context.detail
    request.response.status_int = 404
    return err_dict(message)


def forbidden(request):
    log.info("error.forbidden")
    if unauthenticated_userid(request):
        request.response.status_int = 403
        return err_dict('You are not allowed to perform this action.')
    else:
        request.response.status_int = 401
        return err_dict('You must login to perform this action.')



import os
import unittest
import inflection

from pyramid import testing

from pyramid.config import Configurator

from .base import DATA_DIR
from .resource import BOOKS


class ResourceFunctionalTests(unittest.TestCase):

    def setUp(self):
        settings = {
            'pyramlson.apidef_path': os.path.join(DATA_DIR, 'test-api.raml'),
            'pyramlson.debug': 'true',
            'pyramlson.arguments_transformation_callback': inflection.underscore
        }
        self.config = testing.setUp(settings=settings)
        self.config.include('pyramlson')
        self.config.scan('.resource')
        from webtest import TestApp
        self.testapp = TestApp(self.config.make_wsgi_app())

    def tearDown(self):
        testing.tearDown()

    def test_get_list_json(self):
        r = self.testapp.get('/api/v1/books', status=200)
        assert r.json_body == list(BOOKS.values())

    def test_get_one(self):
        app = self.testapp
        r = app.get('/api/v1/books/123', status=200)
        assert r.json_body == BOOKS[123]
        r = app.get('/api/v1/books/456', status=200)
        assert r.json_body == BOOKS[456]

    def test_get_notfound(self):
        app = self.testapp
        r = app.get('/api/v1/books/111', status=404)
        assert r.json_body['success'] == False
        assert r.json_body['message'] == "Book with id 111 could not be found."

        book_id = 10
        fake_book = {'id': book_id, 'title': 'Foo', 'author': 'Blah'}
        r = app.put_json('/api/v1/books/{}'.format(book_id), params=fake_book, status=404)
        assert r.json_body['success'] == False
        assert r.json_body['message'] == "Book with id {} could not be found.".format(book_id)

    def test_get_general_error(self):
        app = self.testapp
        r = app.get('/api/v1/books/zzz', status=400)
        assert r.json_body['success'] == False
        assert r.json_body['message'] == "Malformed parameter 'bookId', expected integer, got 'zzz'"

    def test_json_validation_error(self):
        app = self.testapp
        r = app.put('/api/v1/books/111', status=400)
        assert r.json_body['message'] == "Empty body!"
        assert r.json_body['success'] == False

        r = app.request('/api/v1/books/111',
            method='PUT',
            body=b'{',
            status=400,
            content_type='application/json')
        assert r.json_body['success'] == False
        assert str(r.json_body['message']).startswith("Invalid JSON body:")

        book_id = 10
        fake_book = {'author': 'Blah'}
        r = app.put_json('/api/v1/books/{}'.format(book_id), params=fake_book, status=400)
        assert r.json_body['success'] == False
        assert 'Failed validating' in r.json_body['message']

    def test_not_accepted_body_mime_type(self):
        app = self.testapp
        r = app.request('/api/v1/books/123',
            method='PUT',
            body=b'hi there',
            status=400,
            content_type='text/plain')
        assert r.json_body['success'] == False
        assert "Invalid JSON body:" in r.json_body['message']

    def test_succesful_json_put(self):
        app = self.testapp
        book_id = 123
        fake_book = {'id': book_id, 'title': 'Foo', 'author': 'Blah'}
        r = app.put_json('/api/v1/books/{}'.format(book_id), params=fake_book, status=200)
        assert r.json_body['success'] == True


    def test_default_options(self):
        app = self.testapp
        r = app.options('/api/v1/books', status=204)
        header = r.headers['Access-Control-Allow-Methods'].split(', ')
        assert 'POST' in header
        assert 'GET' in header
        assert 'OPTIONS' in header

    def test_required_uriparams(self):
        app = self.testapp
        tt = 'a'
        r = app.get('/api/v1/books/some/other/things', params=dict(thingType=tt), status=200)
        # note the renamed argument
        assert r.json_body['thing_type'] == tt

    def test_missing_required_uriparams(self):
        app = self.testapp
        tt = 'a'
        r = app.get('/api/v1/books/some/other/things', params=dict(foo='bar'), status=400)
        assert r.json_body['message'] == 'thingType (string) is required'

class NoMatchingResourceMethodTests(unittest.TestCase):

    def setUp(self):
        settings = {
            'pyramlson.apidef_path': os.path.join(DATA_DIR, 'test-api.raml'),
        }
        self.config = testing.setUp(settings=settings)

    def test_valueerror(self):
        self.config.include('pyramlson')
        self.assertRaises(ValueError, self.config.scan, '.bad_resource')



# coding: utf-8
"""
Pyramlson utilities
"""
import jsonschema

from pyramid.httpexceptions import HTTPBadRequest
from pyramid.renderers import render_to_response

from .apidef import IRamlApiDefinition


def prepare_json_body(request, body):
    """ Convert request body to json and validate it. """
    if not request.body:
        raise HTTPBadRequest(u"Empty body!")
    try:
        data = request.json_body
    except ValueError:
        raise HTTPBadRequest(u"Invalid JSON body: {}".format(request.body))
    apidef = request.registry.queryUtility(IRamlApiDefinition)
    schema = apidef.get_schema(body)
    if schema:
        try:
            jsonschema.validate(
                data,
                schema,
                format_checker=jsonschema.draft4_format_checker
            )
        except jsonschema.ValidationError as err:
            raise HTTPBadRequest(str(err))
    return data


def render_view(request, data, status_code):
    """ Render data to response using the correct response status code """
    response = request.response
    response.status_int = status_code
    try:
        return render_to_response('json', data, request=request, response=response)
    except TypeError:
        # 1.5.7 compat
        return render_to_response('json', data, request=request)



import os
import sys
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst')) as f:
    README = f.read()
with open(os.path.join(here, 'CHANGES.txt')) as f:
    CHANGES = f.read()

VERSION = CHANGES.splitlines()[0]

install_requires = [
    'pyramid',
    'jsonschema',
    'ramlfications>=0.1.9',
]

tests_require = install_requires + [
    'pytest',
    'WebTest',
    'inflection'
]
testing_extras = tests_require + [
    'nose',
    'coverage',
]

setup(name='pyramlson',
    version=VERSION,
    description='pyramlson',
    long_description=README + '\n\n' + CHANGES,
    classifiers=[
        "Programming Language :: Python",
        "Framework :: Pyramid",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    author='Igor Stroh',
    author_email='igor.stroh@rulim.de',
    url='http://github.com/jenner/pyramlson',
    keywords='web pyramid api json',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    tests_require=tests_require,
    extras_require = {
        'testing': testing_extras,
    },
    test_suite="pyramlson",
)



# coding: utf-8
"""
Pyramlson API Definition utility
"""
import ramlfications

from zope.interface import Interface

try:
    from urllib.parse import urlparse
except ImportError: # pragma: no cover
    from urlparse import urlparse


class IRamlApiDefinition(Interface):
    """ Marker interface for API Definition """
    # pylint: disable=inherit-non-class
    pass


class RamlApiDefinition(object):
    """ RAML Definition utility.
        Abstracts the access to parsed RAML data
    """

    __traits_cache = {}

    def __init__(self, apidef_path, args_transform_cb=None):
        self.raml = ramlfications.parse(apidef_path)
        self.base_uri = self.raml.base_uri
        if self.base_uri.endswith('/'):
            self.base_uri = self.base_uri[:-1]
        self.base_path = urlparse(self.base_uri).path
        self.args_transform_cb = args_transform_cb

    @property
    def default_mime_type(self):
        """ Return the default mime-type for a resource
            if none was defined in RAML
        """
        return self.raml.media_type

    def get_trait(self, name):
        """ Return a trait from RAML """
        if not self.raml.traits:
            return None
        trait = None
        if name not in self.__traits_cache:
            for trait in self.raml.traits:
                if trait.name == name:
                    self.__traits_cache[name] = trait
        return self.__traits_cache.get(name)

    def get_resources(self, path=None):
        """ Get resources """
        if not path:
            return self.raml.resources
        return (res for res in self.raml.resources if res.path == path)

    def get_schema_def(self, name):
        """ Get schema definition """
        if self.raml.schemas is None:
            return None
        for schemas in self.raml.schemas:
            if name in schemas:
                return schemas[name]

    def get_schema(self, body):
        """ Extract a schema from body for a given mime-type """
        if isinstance(body, list):
            bodies = body
            for body in bodies:
                if body.mime_type == 'application/json':
                    break
        if not body or not body.schema:
            return None
        schema = body.schema
        # FIXME there should be a better way to detect an inline schema
        if '$schema' not in schema:
            schema = self.get_schema_def(schema)
        return schema
